# coding: utf-8
u"""JSON 関連のユーティリティ。
Python のバージョン間で JSON ライブラリの有無が違うことへの対処、
JSON オブジェクトへのスキーマ適用などの処理などを行う。
"""

try:
    import json
except:
    try:
        import simplejson as json
    except:
        raise ImportError('Python 2.6 (or later) or simplejson package is needed')

from caty.jsontools.model import *
__all__ = ['load', 'loads', 'dump', 'dumps', 'prettyprint', 'pp']

def load(fo):
    u"""ファイル類似オブジェクトより JSON データをロードする。
    """
    return auto_tag(json.load(fo))

def loads(s):
    u"""文字列より JSON データをロードする。
    """
    return auto_tag(json.loads(s))

def auto_tag(v):
    if '$tag' in v:
        if '$val' in v:
            return taged(v['$tag'], v['$val'])
        else:
            t = v['$tag']
            del v['$atg']
            return tagged(t, v)
    else:
        return v

def dump(obj, fo):
    return json.dump(dump_tag(obj), fo)

def dumps(obj):
    return json.dumps(dump_tag(obj))

def dump_tag(v):
    if isinstance(v, TaggedValue):
        return v.to_dict()
    else:
        return v

def prettyprint(obj, depth=0):
    def format(o):
        if isinstance(o, TaggedValue):
            return '@%s %s' % (tag(o), prettyprint(untagged(o)))
        elif isinstance(o, unicode):
            return u'"%s"' % o
        elif isinstance(o, list):
            return u'[%s]' % (', '.join(map(prettyprint, o)))
        elif o == None:
            return u'null'
        elif isinstance(o, bool):
            return str(o).lower()
        else:
            return str(o)
    if not isinstance(obj, dict):
        return format(obj)
    r = [u'{\n']
    values = []
    depth += 1
    for k, v in sorted(obj.iteritems()):
        space = u'    ' * depth
        val = format(v)
        values.append('%s"%s":%s' % (space, k, val))
    r.append(',\n'.join(values))
    r.append(u'    ' * (depth - 1))
    r.append(u'\n}\n')
    return u''.join(r)

pp = prettyprint

