# coding: utf-8
from caty.jsontools.path.JsonPathParser import Parser, JsonPathError, undefined

class JsonPathQuery(Parser):
    def __init__(self, *args, **kwds):
        Parser.__init__(self, *args, **kwds)
        self.query = []

    def _get_value(self, jsonobj, query):
        if isinstance(jsonobj, list):
            try:
                return jsonobj[int(query)]
            except:
                return undefined
        elif isinstance(jsonobj, dict):
            return jsonobj.get(query, undefined)
        else:
            JsonPathError("list or dict is required")

    def _find_wildcard_list(self, jsonobj, query, error_if_not_found=False):
        if not isinstance(jsonobj, list):
            raise JsonPathError("list instance is required")
        for v in jsonobj:
            yield self.find(v, query[1:], error_if_not_found)

    def _find_wildcard_obj(self, jsonobj, query, error_if_not_found=False):
        if not isinstance(jsonobj, dict):
            raise JsonPathError("dict instance is required")
        for v in jsonobj.values():
            yield self.find(v, query[1:], error_if_not_found)

    def _find(self, jsonobj, query, error_if_not_found=False):
        q = query[0]
        v = self._get_value(jsonobj, q)
        if v == undefined:
            if error_if_not_found:
                raise JsonPathError("Value not found: %s" % q)
            else:
                yield None
        else:
            if query[1:]:
                for r in self.find(v, query[1:], error_if_not_found):
                    yield r
            else:
                yield v

    def find(self, jsonobj, query=None, error_if_not_found=False):
        if query == None:
            return self.find(jsonobj, self.query[1:], error_if_not_found)
        elif query == []:
            return iter([jsonobj])
        else:
            q = query[0]
            if q == '*':
                return self._find_wildcard_obj(jsonobj, query[1:], error_if_not_found)
            elif q == '#':
                return self._find_wildcard_list(jsonobj, query[1:], error_if_not_found)
            else:
                return self._find(jsonobj, query, error_if_not_found)

    def _root(self, r):
        if self.query != []:
            raise JsonPathError("JsonPath must starts with \"$\"")
        self.query.append(r.getText())

    def _path(self, p):
        if self.query == []:
            raise JsonPathError("JsonPath must starts with \"$\"")
        self.query.append(p.getText())


