# coding: utf-8
import sys
import os
sys.path.insert(0, os.path.abspath('../../../../'))
from caty.testutil import *
from caty.jsontools.path.JsonPathLexer import *
from caty.jsontools.path.query import *
from caty.jsontools.model import JsonModel
from StringIO import StringIO

def mk_lexer(s):
    io = StringIO()
    io.write(s)
    io.seek(0)
    return Lexer(io)

def mk_query(s):
    l = mk_lexer(s)
    p = JsonPathQuery(l)
    p.parse_jsonpath()
    return p

class QueryTest(TestCase):
    def test_find(self):
        jo = JsonModel({'foo':'foo!!', 'bar': {'buz':'buz!!'}})
        query = mk_query('$.foo')
        v = list(query.find(jo))[0]
        self.assertEquals('foo!!', v)

        query = mk_query('$.bar.buz')
        v = list(query.find(jo))[0]
        self.assertEquals('buz!!', v)

    def test_find_list(self):
        jo = JsonModel({'foo':[1,2,3]})
        query = mk_query('$.foo.#')
        v = list(query.find(jo))
        self.assertEquals([1,2,3], v)

    def test_find_objmember(self):
        jo = JsonModel({'foo':{'a': 1, 'b': 2, 'c': 3}})
        query = mk_query('$.foo.*')
        v = list(sorted(query.find(jo)))
        self.assertEquals([1,2,3], v)

    def test_find_obj_root(self):
        jo = JsonModel({'foo':{'a': 1, 'b': 2, 'c': 3}})
        query = mk_query('$')
        v = query.find(jo).next()
        self.assertEquals(jo, v)


if __name__ == '__main__': 
    test(QueryTest)

