#coding:utf-8
import re
from topdown import *
from caty.jsontools.schema.language.ast import *
from caty.jsontools.schema.language.util import *
from caty.jsontools.schema.language.schemaparser import term, typedef, string, value

def command(seq):
    doc = option(docstring)(seq)
    _ = seq.parse('command ')
    n = seq.parse(cmdname)
    patterns = many1(call_pattern)(seq)
    r = seq.parse(many1(refers))
    _ = seq.parse(';')
    return CommandNode(n, patterns, r, doc)

def cmdname(seq):
    return seq.parse(Regex(r'[a-zA-Z_](-?[a-zA-Z0-9_]*)*'))

def call_pattern(seq):
    o = option(cmdoption)(seq)
    if o:
        opt = ObjectNode(o)
    else:
        opt = None
    a = option(cmdargument)(seq)
    if a:
        arg = ScalarNode('array')
        arg.underlyingtypes = a
    else:
        arg = None
    _ = seq.parse('::')
    d = decl(seq)
    return CallPattern(opt, arg, d)

def cmdoption(seq):
    _ = seq.parse('{')
    o = option(chain_flat(optitem, comma), [])(seq)
    _ = seq.parse('}')
    return dict(o)

def optitem(seq):
    k = seq.parse(string)
    _ = seq.parse(':')
    s = seq.parse([mini_enum, term])
    return [k, s]

def enum_delim(seq):
    _ = seq.parse('|')
    return _

def mini_enum(seq):
    r = chain_flat(value, enum_delim)(seq)
    return EnumNode(r)

def cmdargument(seq):
    _ = seq.parse('[')
    o = option(chain_flat([term, mini_enum], comma), [])(seq)
    _ = seq.parse(']')
    return o

def decl(seq):
    p = seq.parse(profile)
    j = seq.parse(many(jump))
    r = seq.parse(many(resource))
    return CommandDecl(p, j, r)

def profile(seq):
    def _profile(s):
        i = typedef(s)
        _ = s.parse('->')
        o = typedef(s)
        return i, o
    return chain_flat(_profile, comma)(seq)

def jump(seq):
    return seq.parse([throws, breaks])

def throws(seq):
    _ = seq.parse('throws ')
    return _, seq.parse([lambda s: [typedef(s)], typelist])

def breaks(seq):
    _ = seq.parse('breaks ')
    return _, seq.parse([lambda s: [typedef(s)], typelist])

def typelist(seq):
    _ = seq.parse('[')
    r = chain_flat(typedef, comma)(seq)
    _ = seq.parse(']')
    return r

def namelist(seq):
    _ = seq.parse('[')
    r = chain_flat(name, comma)(seq)
    _ = seq.parse(']')
    return r

def resource(seq):
    return seq.parse([uses, reads, updates])

def updates(seq):
    _ = seq.parse('uses ')
    return _, seq.parse([lambda s:[name(s)], namelist])

def reads(seq):
    _ = seq.parse('reads ')
    return _, seq.parse([lambda s:[name(s)], namelist])

def uses(seq):
    _ = seq.parse('updates ')
    return _, seq.parse([lambda s:[name(s)], namelist])

def refers(seq):
    _ = seq.parse('refers ')
    l = seq.parse(Regex(r'[a-zA-Z]+:'))
    return l, seq.parse(Regex(r'([a-zA-Z][a-zA-Z0-9]*(\.[a-zA-Z][a-zA-Z0-9]*)*)'))

