header {
from caty.jsontools.schema.language.ast import *
}
options {
    language = "Python";
}

class JsonSchemaParser extends Parser;
    parse_schema returns [r]:
        {r=[]}
        ("type" t:SYMBOL EQUAL s=schema_def DELIM{
            r.append(RootNode(t, s))
        })*;
    schema_def returns [r]:
        TAG t:SYMBOL d=schema_def {
            r=TaggedNode(t, d)
        }
        |
        {s=None; c=None}
        s=schema_def2 (c=combined_schemadef)? 
        {
            if not c:
                r = s
            else:
                r = c[0](s, c[1])
        };
    combined_schemadef returns [r]:
        {o=None; a=None; s=None; c=None}
        (o:OR|a:AND) s=schema_def2 (c=combined_schemadef)? {
            if o:
                combine = UnionNode
            else:
                combine = IntersectionNode
            if not c:
                r = combine, s
            else:
                c2, s2 = c
                newnode = c2(s, s2)
                r = combine, newnode
        };
    schema_def2 returns [r]:
        {oi = []}
        ("object" LBRACE x=objitem{oi.append(x)} 
                (COMMA o=objitem {oi.append(o)})* RBRACE op=optional{
            items=dict(oi)
            if "*" in items:
                wildcard=items["*"]
                items.pop("*")
            else:
                wildcard=None
            o = ObjectNode(items, wildcard)
            if op:
                o.optional = True
            r=o
        })|(
        s:SYMBOL o=option u=underlyingtypes op=optional{
            r = ScalarNode(s.getText())
            r.options = o if o else {}
            udt = []
            for s, o in u:
                if o:
                    r.options["repeat"] = True
                udt.append(s)
            r.underlyingtypes = udt
            if op:
                r.optional = True
        });
    objitem returns [r]:
        (a:ASTER|k:STRING) COLON s=schema_def {
            r = k.getText() if k else a.getText(), s
        }
    ;
    optional returns [r]:
        {r=False}
        (o:OPTIONAL {
            r = True if o else False
        })?;
    option returns [r]:
        {r=[]}
        (LPAREN o=optvalue {r.append(o)} (COMMA v=optvalue {r.append(v)})* RPAREN {
        })?;
    underlyingtypes returns [r]:
        {r=[]}
        (LBRACKET a=scalar {r.append(a)} (COMMA t=scalar{r.append(t)})* RBRACKET)?;
    scalar returns [r]:
        s:SYMBOL (a:ASTER|o:OPTIONAL)? {
            x = ScalarNode(s.getText())
            if o:
                x.optional = True
            r = x, a
        };

    optvalue returns [r]:
         k:SYMBOL EQUAL (v:STRING | i:NUMBER) {
             r = (k.getText(), v.gettext() if v else int(i.getText()))
         };

class JsonSchemaLexer extends Lexer;

options {
    k=2;
    filter=true;
    charVocabulary = '\u0000'..'\uFFFE';
}

INLINECOMMENT: "//" (~'\n')* {$skip};
MULTILINECOMMENT: "/*" ('\u0000'..'\uFFFE') "*/" {$skip};
COLON : ':';
STRING : '"'! (~'"')* '"'!;
SYMBOL : ((TOKEN ('.' TOKEN)* (':' TOKEN)?));
protected
TOKEN : ('a'..'z'|'A'..'Z'|'$') ('a'..'z'|'A'..'Z'|'0'..'9')*;
LPAREN : '(';
RPAREN : ')';
LBRACE : '{';
RBRACE : '}';
LBRACKET : '[';
RBRACKET : ']';
WS : (' '|'\t'|'\n') { $skip };
COMMA : ',';
EQUAL : '=';
DELIM : ';';
OR : '|';
AND : '&';
TAG : '@'! TOKEN;
ASTER : '*';
OPTIONAL : '?';
NUMBER: ('0')|(('1'..'9')('0'..'9')*('.' ('0'..'9')('0'..'9')*)?);
