#coding:utf-8
from topdown import *
from caty.jsontools.schema.language.ast import *
from caty.jsontools.schema.language.util import *
from caty.jsontools.schema.schematypes import schemas

RESERVED = frozenset(schemas.keys())

def schema(seq):
    t = seq.parse('type ')
    n = seq.parse(name)
    if name in RESERVED:
        raise ParseFailed('%s is reserved.' % n)
    e = seq.parse('=')
    d = seq.parse(typedef)
    c = seq.parse(';')
    return SchemaNode(n, d)


def typedef(seq):
    return chainl(term, op)(seq)

def op(seq):
    o = seq.parse(['&', '|'])
    if o == '&':
        return IntersectionNode
    else:
        return UnionNode

def term(seq):
    def _tag(s):
        _ = s.parse('@')
        t = s.parse(name)
        v = s.parse(term)
        return TaggedNode(t, v)
    def _term(s):
        _ = s.parse('(')
        d = s.parse(typedef)
        _ = s.parse(')')
        return d
    s = seq.parse([_tag, enum, _term, object_, scalar])
    o = seq.parse(option('?'))
    if o:
        s.optional = True
    return s

def scalar(seq):
    n = seq.parse(typename)
    o = seq.parse(option(options, {})) # dictionary
    t = seq.parse(option(underlyingtypes, [])) # list of ScalarNode
    udt = []
    node = ScalarNode(n)
    node.options = o
    for s, o in t:
        if o == '*':
            if 'repeat' in s.options and node.options['repeat']:
                raise ParseFailed(seq, scalar)
            node.options['repeat'] = True
        elif o == '?':
            s.optional = True
        udt.append(s)
    node.underlyingtypes = udt if udt else None
    return node

def typename(seq):
    return seq.parse(Regex(r'([a-zA-Z_][a-zA-Z0-9_]*(\.[_a-zA-Z][_a-zA-Z0-9]*)*\:)?(([_a-zA-Z][_a-zA-Z0-9]*)|(\$))'))

def options(seq):
    _ = seq.parse('(')
    opts = option(chain_flat(optval, comma))(seq)
    _ = seq.parse(')')
    if not opts: return {}
    return dict(opts)

def optval(seq):
    k = seq.parse(name)
    _ = seq.parse('=')
    v = seq.parse(value)
    return [k, v]

def value(seq):
    return seq.parse([string, boolean, integer, number])

def string(seq):
    _ = seq.parse('"')
    s = seq.parse(until('"'))
    _ = seq.parse('"')
    return s

def boolean(seq):
    return bool(seq.parse(['true', 'false']).title())

def integer(seq):
    return int(seq.parse(Regex(r'-?[0-9]+')))

def number(seq):
    return int(seq.parse(Regex(r'-?[0-9]+\.[0-9]+')))

def underlyingtypes(seq):
    seq.parse('[')
    r = chain_flat(repeatable_type, comma)(seq)
    seq.parse(']')
    return r

def repeatable_type(seq):
    s = seq.parse(term)
    r = seq.parse(option(['*', '?']))
    return [s, r]

def enum(seq):
    _ = seq.parse('enum')
    _ = seq.parse('[')
    enum = chain_flat(value, comma)(seq)
    _ = seq.parse(']')
    if not all(map(lambda e: e.__class__ == enum[0].__class__, enum)):
        raise ValueError(str(enum))
    return EnumNode(enum)

def object_(seq):
    o = seq.parse('object')
    _ = seq.parse(option('{'))
    if not _:
        return ObjectNode({})
    items = dict(chain_flat(item, comma)(seq))
    _ = seq.parse('}')
    if '*' in items:
        wildcard = items['*']
        items.pop('*')
    else:
        wildcard = None
    return ObjectNode(items, wildcard)

def item(seq):
    k = seq.parse([string, '*'])
    _ = seq.parse(':')
    s = typedef(seq)
    return [k, s]

