# coding: utf-8
import sys
import os
from StringIO import StringIO
sys.path.insert(0, os.path.abspath('../../../../../'))
from caty.jsontools.schema.language.JsonSchemaLexer import *
from caty.testutil import TestCase, test

def lex(s):
    io = StringIO()
    io.write(s)
    io.seek(0)
    lexer = Lexer(io)
    r = []
    for t in lexer:
        r.append((t.getType(), t.getText()))
    return r

class LexerTest(TestCase):
    def test_lex(self):
        self.assertEquals(lex('string'), [(SYMBOL, 'string')])
        self.assertEquals(lex('"name"'), [(STRING, 'name')])
        self.assertEquals(lex('foo:buz'), [(SYMBOL, 'foo:buz')])
        self.assertEquals(lex('foo.bar:buz'), [(SYMBOL, 'foo.bar:buz')])
        self.assertEquals(lex('{ "name" : bool}'), [(LBRACE, '{'), (STRING, 'name'), (COLON, ':'), (SYMBOL, 'bool'), (RBRACE, '}')])
        self.assertEquals(lex("""
        udit = object { 
            "name1" : bool, 
            "name2" : array [string]
        }"""), [(SYMBOL, 'udit'), (EQUAL, '='), (SYMBOL, 'object'),
        (LBRACE, '{'), 
            (STRING, 'name1'), (COLON, ':'), (SYMBOL, 'bool'), (COMMA, ','),
            (STRING, 'name2'), (COLON, ':'), (SYMBOL, 'array'), (LBRACKET, '['), (SYMBOL, 'string'), (RBRACKET, ']'),
        (RBRACE, '}')])
        self.assertEquals(lex('type foo = bar:$;'), [(SYMBOL, 'type'), (SYMBOL, 'foo'), (EQUAL, '='), (SYMBOL, 'bar:$'), (DELIM, ';')])

if __name__ == '__main__':
    test(LexerTest)

