# coding: utf-8
import sys
import os
from StringIO import StringIO
sys.path.insert(0, os.path.abspath('../../../../../'))
from caty.jsontools.schema.language.casmparser import parse
from caty.jsontools.schema.language.util import SchemaSyntaxError
from caty.jsontools.schema.language.ast import *
from caty.testutil import TestCase, test

class CommandParserTest(TestCase):
    def test_simple_decl(self):
        s = 'command foo {} [] :: string -> string refers python:Command;'
        self.assertNotRaises(parse, s)
        s = 'command foo :: string -> string refers python:foo.bar.Command;'
        self.assertNotRaises(parse, s)
        s = '/** doc string */\ncommand foo :: string -> string refers python:foo.bar.Command;'
        self.assertNotRaises(parse, s)
        s = '/* comment */\ncommand foo {} [] :: object -> object refers python:foo.bar.Command;'
        self.assertNotRaises(parse, s)

    def test_decl_with_opts(self):
        s = 'command foo {"x":integer} [] :: string -> string refers python:foo.bar.Command;'
        self.assertNotRaises(parse, s)

        s = 'command foo {"y":1|2|3} [] :: string -> string refers python:foo.bar.Command;'
        self.assertNotRaises(parse, s)

        s = 'command foo {"x":integer, "y":string?} [] :: string -> string refers python:foo.bar.Command;'
        self.assertNotRaises(parse, s)

        s = 'command foo {"x":integer, "hjkl":string?} [] :: string -> string refers python:foo.bar.Command;'
        self.assertNotRaises(parse, s)

    def test_multi_decl(self):
        s = 'command foo {"x":integer} [] :: string -> string refers python:foo.bar.Command;'
        s = s + 'command bar :: string -> string refers python:foo.bar.Command2;'
        self.assertNotRaises(parse, s)

    def test_overload(self):
        s = 'command foo {"x":integer} [] :: string -> string'
        s = s + ', int -> string refers python:foo.bar.Command;'
        self.assertNotRaises(parse, s)
        
    def test_jump_decl(self):
        s = """command foo :: string -> string
        throws Error
        refers python:foo.bar.Command;
        """
        self.assertNotRaises(parse, s)

        s = """command foo :: string -> string
        throws [Error1, Erro2]
        refers python:foo.bar.Command;
        """
        self.assertNotRaises(parse, s)

        s = """command foo :: string -> string
        breaks Ret
        refers python:foo.bar.Command;
        """
        self.assertNotRaises(parse, s)

        s = """command foo :: string -> string
        breaks [Ret1, Ret2]
        refers python:foo.bar.Command;
        """
        self.assertNotRaises(parse, s)

        s = """command foo :: string -> string
        breaks Ret
        throws Error
        refers python:foo.bar.Command;
        """
        self.assertNotRaises(parse, s)

    def test_resource_decl(self):
        s = """command foo :: string -> string
        uses mafs
        refers python:foo.bar.Command;
        """
        self.assertNotRaises(parse, s)

        s = """command foo :: string -> string
        reads [mafs, env]
        refers python:foo.bar.Command;
        """
        self.assertNotRaises(parse, s)


        s = """command foo :: string -> string
        reads mafs
        updates env
        refers python:foo.bar.Command;
        """
        self.assertNotRaises(parse, s)

    def test_heavy_overload(self):
        s = """command buz
        :: string -> string
            reads mafs
            updates env
        
        [string, integer] :: null -> string
            updates [mafs, env]

        {"x":integer} [string] :: null -> string
            updates [mafs, env]
        refers python:foo.bar.Command3;
        """
        self.assertNotRaises(parse, s)

    def test_with_docstring(self):
        s = """/** 
            documentation comment 
        */
        command foo {} [] :: string -> string refers python:foo.bar.Command;"""
        self.assertNotRaises(parse, s)
        
