# coding: utf-8
import sys
import os
sys.path.insert(0, os.path.abspath('../../../'))
import unittest
from caty.jsontools.schema import *
from caty.testutil import *

class IntegerSchemaTest(TestCase):
    def test_validate_ok(self):
        scm = NumberSchema(is_integer=True)
        self.assertNotRaises(scm.validate, 0)
        self.assertNotRaises(scm.validate, 1)
        self.assertNotRaises(scm.validate, -1)

    def test_validate_less_than_minimum(self):
        scm = NumberSchema(is_integer=True, minimum=10)
        self.assertRaises(JsonSchemaError, scm.validate, 9)
        try:
            scm.validate(10)
        except Exception, e:
            self.fail(e)
    
    def test_validate_greater_than_maximum(self):
        scm = NumberSchema(is_integer=True, maximum=10)
        self.assertRaises(JsonSchemaError, scm.validate, 11)
        try:
            scm.validate(10)
        except Exception, e:
            self.fail(e)

    def test_type_error(self):
        scm = NumberSchema(is_integer=True)
        self.assertRaises(JsonSchemaError, scm.validate, '1')
        self.assertRaises(JsonSchemaError, scm.validate, 1.0)
        self.assertRaises(JsonSchemaError, scm.validate, True)

    def test_convert(self):
        scm = NumberSchema(is_integer=True, maximum=10)
        v = scm.convert('1')
        self.assertEquals(1, v)
        self.assertRaises(ValueError, scm.convert, 'a')
        self.assertRaises(JsonSchemaError, scm.convert, '12')

        
