#coding:utf-8
from __future__ import with_statement
from caty.jsontools.schema.schematypes import schemas
from caty.jsontools.schema.language.casmparser import iparse
from caty.jsontools.schema.language.ast import SchemaNode, CommandNode, Provide
from caty.jsontools.path.validator import to_decl_style
from threading import RLock
from StringIO import StringIO

search_path = '/schema' # スキーマ定義のサーチパス。デフォルトから変更の必要はない?

class Registrar(object):
    u"""JSON スキーマの型レジストリ。
    """
    schema_ns = schemas # 組み込みスキーマへの参照
    command_ns = {} # 組み込みコマンドの型宣言への参照
    fs = None
    search_path = ''
    module_ns = {} # モジュール名称: Module の辞書

    @classmethod
    def get_instance(cls, fs=None, search_path='/schema'):
        if not cls.fs:
            cls.fs = fs
        if not cls.search_path:
            cls.search_path = search_path
        return Registrar()

    def find(self, do):
        for e in do.read():
            if not e.is_dir:
                yield e
            else:
                for se in e.read():
                    yield se
    
    def add_command(self, name, profile):
        if name in self.command_ns:
            raise Exception('Already registered: %s' % name)
        self.command_ns[name] = profile

    def compile_all(self):
        for e in self.find(self.fs.DirectoryObject(self.search_path)):
            m = Module(e.path, self.fs, self)
            self.module_ns[m.name] = m
        if 'public' in self.module_ns:
            self.module_ns['public'].compile()
        for v in [v for k, v in self.module_ns.iteritems() if k != 'public']:
            v.compile()
            
    def has_schema(self, name):
        if name in self.schema_ns:
            return True
        elif 'public' in self.module_ns and self.module_ns['public'].has_schema(name):
            return True
        else:
            if ':' not in name:
                raise KeyError(name)
            m, n = name.split(':')
            if m in self.module_ns and self.module_ns[m].has_schema(n):
                return True
        return False

    def get_schema(self, name):
        if name in self.schema_ns:
            return self.schema_ns[name]
        elif 'public' in self.module_ns and self.module_ns['public'].has_schema(name):
            return self.module_ns['public'].get_schema(name)
        else:
            if ':' not in name:
                raise KeyError(name)
            m, n = name.split(':')
            if m in self.module_ns: #and self.module_ns[m].has_schema(n):
                return self.module_ns[m].get_schema(n)
        raise KeyError(name)

    def get_command_type(self, name):
        if name in self.command_ns:
            return self.command_ns[name]
        for m in self.module_ns.values():
            if m.has_command(name):
                return m.get_command(name)
        raise KeyError(name)


    def compile(self, schema_string):
        u"""組み込みコマンドの型指定など、オンザフライでスキーマを構築するためのメソッド。
        """
        io = schema_string#StringIO(schema_string)
        for t in iparse(io):
            t.build_schema_tree(self)

    @property
    def command_profiles(self):
        for k, v in self.command_ns.iteritems():
            yield k, v
        for m in self.module_ns.values():
            for a, b in m.command_profiles:
                yield a, b

class Module(object):
    u"""JSON スキーマとコマンド宣言をまとめたオブジェクト。
    """
    def __init__(self, filepath, mafs, registrar):
        self.filepath = filepath
        self.name = self._path_to_module(filepath)
        self.registrar = registrar
        self.mafs = mafs
        self.schema_ns = {}
        self.command_ns = {}
        self.pcasm_cache = None
        self.parser_cache = None
        self.exports = []

    def add_schema(self, name, member):
        self.schema_ns[name] = member
    
    def add_command(self, name, member):
        self.command_ns[name] = member

    def has_schema(self, name):
        if name in self.schema_ns:
            if not self.exports or name in self.exports:
                return True
        return False

    def has_command(self, name):
        if name in self.command_ns:
            if not self.exports or name in self.exports:
                return True
        return False

    def get_schema(self, name, tried_to_compile=False):
        if name in self.schema_ns:
            return self.schema_ns[name]
        else:
            try:
                return self.registrar.get_schema(name)
            except KeyError, e:
                if not tried_to_compile:
                    self.compile()
                    return self.get_schema(name, True)
        raise KeyError(name)

    def get_command(self, name, tried_to_compile=False):
        if name in self.command_ns:
            return self.command_ns[name]
        else:
            if not tried_to_compile:
                self.compile()
                return self.get_command(name)
        return KeyError(name)

    def _path_to_module(self, path):
        p = path.replace(search_path, '')[1:].rsplit('.', 1)[0]
        return p.replace('/', '.')

    def _module_to_path(self, modname):
        m = modname.split(':')[0]
        return '/' + m.replace('.', '/')

    def compile(self):
        o = self.mafs.open(self.filepath)
        if self.filepath.endswith('.casm'):
            for t in self.find_or_create_cache(o):
                t.build_schema_tree(self)
        elif self.filepath.endswith('.pcasm'):
            d = to_decl_style(o)
            if self.pcasm_cache:
                io = self.pcasm_cache
            else:
                io = StringIO(d)
                self.pcasm_cache = io
            for t in self.find_or_create_cache(io):
                t.build_schema_tree(self)
        else:
            #raise Exception('Invalid file name. %s' % self.filepath)
            pass
            #raise Exception('Invalid file name. %s' % path)

    def find_or_create_cache(self, fo):
        if not self.parser_cache:
            self.parser_cache = iparse(fo.read())
        return self.parser_cache

    @property
    def command_profiles(self):
        for k, v in self.command_ns.iteritems():
            yield k, v
