# coding: utf-8
import sys
import os
sys.path.insert(0, os.path.abspath('../../../'))
from caty.jsontools.model import JsonModel, to_tagged_value, TaggedValue
from caty.testutil import TestCase, test

class JsonModelTest(TestCase):
    def testInitEmpty(self):
        j = JsonModel()
        self.assertEquals(len(j.keys()), 0)
        self.assertEquals(j, {})

    def testInitDict(self):
        j = JsonModel({'x': 'y', 'a': 'b'})
        self.assertEquals(len(j.keys()), 2)
        self.assertEquals(j, {'x': 'y', 'a': 'b'})

    def testInitTuple(self):
        j = JsonModel([('x', 'y'), ('a', 'b')])
        self.assertEquals(len(j.keys()), 2)
        self.assertEquals(j, {'x': 'y', 'a': 'b'})

    def testInitKArg(self):
        j = JsonModel(x='y', a='b')
        self.assertEquals(len(j.keys()), 2)
        self.assertEquals(j, {'x': 'y', 'a': 'b'})

    def testSetAndGet(self):
        j = JsonModel()
        j['key1'] = 'value'
        j.key2 = 0
        self.assertEquals(len(j.keys()), 2)
        self.assertEquals(j.key1, 'value')
        self.assertEquals(j['key2'], 0)

    def testFailGet(self):
        j = JsonModel()
        try:
            v = j['unknown']
        except KeyError, e:
            pass
        except:
            self.fail()
        else:
            self.fail()

    def testUpdateDict(self):
        j = JsonModel()
        d = {'x': 'y'}
        j.update(d)
        self.assertEquals(j, d)

    def testUpdateJson(self):
        j = JsonModel()
        d = JsonModel({'x': 'y'})
        j.update(d)
        self.assertEquals(j, d)

class TaggedValueTest(TestCase):
    def test_init(self):
        v = to_tagged_value('foo', [1,2,3])
        self.assertTrue(isinstance(v, list))
        self.assertTrue(isinstance(v, TaggedValue))
        self.assertEquals([1,2,3], v)
        self.assertEquals('foo', v.tag)

    def test_json_model(self):
        v = {'$tag': 'foo', '$val': 1}
        j = JsonModel()
        j.x = v
        self.assertTrue(isinstance(j.x, int))
        self.assertTrue(isinstance(j.x, TaggedValue))
        self.assertEquals(1, j.x)
        self.assertEquals('foo', j.x.tag)

if __name__ == '__main__': 
    test(JsonModelTest, TaggedValueTest)


