# coding: utf-8
import os
from caty.mafs.metadata import FileMetadata, DirectoryMetadata, DirectoryEntry, mime_type, caty_dec, xml_dec, xml_dec_with_enc
from caty.mafs.authorization import simple_checker, CREATE, READ, UPDATE, DELETE

def createFile(authori_token, path):
    if path.endswith('/'):
        raise Exception
    os.open(path, os.O_CREAT)

def putFile(authori_token, path, contents):
    if path.endswith('/'):
        raise Exception
    try:
        open(path, 'wb').write(contents)
    except:
        os.remove(path)
        raise

def createDirectory(authori_token, path):
    os.makedirs(path)

def deleteFile(authori_token, path):
    if path.endswith('/'):
        raise Exception
    os.remove(path)

def deleteDirectory(authori_token, path):
    os.rmdir(path)

def delete(authori_token, path):
    if os.path.isdir(path):
        deleteDirectory(authori_token, path)
    else:
        deleteFile(authori_token, path)

def readFile(authori_token, path):
    return open(path, 'rb').read()

def getMetadata(authori_token, path):
    def getFileMetadata():
        tests =[ lambda s: s.endswith('.icaty'),
                 lambda s: s.endswith('.ictpl'),
                 lambda s: s.startswith('.'),
               ]

        basename = os.path.basename(path)
        for test in tests:
            if test(basename):
                hidden = True
                break
        else:
            hidden = False
        ext = os.path.splitext(path)[-1]
        contentType= mime_type(ext)
        st = os.stat(path)
        contentLength = st.st_size
        lastModified = st.st_mtime
        executable = path.endswith('.caty') or path.endswith('.icaty')
        meta = FileMetadata(contentLength, lastModified, contentType, executable, hidden)
        if meta.isText:
            encoding = 'utf-8'
            line = open(path, 'rb').readline()
            if xml_dec.match(line):
                encoding = 'utf-8'
            else:
                dec = xml_dec_with_enc.match(line) or caty_dec.match(line)
                if dec:
                    encoding = dec.group(1).lower()
            meta.textEncoding = encoding
        return meta

    def getDirectoryMetadata():
        st = os.stat(path)
        lastModified = st.st_mtime
        hidden = os.path.basename(os.path.abspath(path))[1].startswith('.')
        meta = DirectoryMetadata(lastModified, hidden)
        return meta

    if os.path.isdir(path):
        return getDirectoryMetadata()
    else:
        return getFileMetadata()

def readDirectory(authori_token, path, metadata=getMetadata):
    u"""ディレクトリエントリを読み込む。
    第三引数の getMetadata はライブラリの都合でたされたものなので、
    利用者は特に指定する必要は無い。
    """
    for e in os.listdir(path):
        yield DirectoryEntry(e, metadata(authori_token, os.path.join(path, e)))

def writeFile(authori_token, path, content):
    if path.endswith('/'):
        raise Exception
    if not os.path.exists(path):
        raise Exception
    open(path, 'wb').write(content)

