# coding: utf-8
import sys
import os
sys.path.insert(0, os.path.abspath('../../../../'))
import unittest
from caty.mafs.stdfs import *
from caty.mafs.metadata import *

class StdFsTest(unittest.TestCase):
    def test_createFile(self):
        createFile(None, 'test.txt')
        self.assertTrue(os.path.exists('test.txt'))
        os.remove('test.txt')

    def test_putFileTese(self):
        putFile(None, 'test.txt', 'aaa')
        self.assertEquals('aaa', open('test.txt').read())
        os.remove('test.txt')

    def test_createDirectory(self):
        createDirectory(None, 'foo')
        self.assertTrue(os.path.exists('foo'))
        self.assertTrue(os.path.isdir('foo'))
        createDirectory(None, 'bar')
        createDirectory(None, 'bar/buz')
        self.assertTrue(os.path.exists('bar/buz'))
        self.assertTrue(os.path.isdir('bar/buz'))
        os.rmdir('foo')
        os.removedirs('bar/buz')

    def test_deleteFile(self):
        open('foo.txt', 'wb').write('a')
        deleteFile(None, 'foo.txt')
        self.assertTrue(not os.path.exists('foo.txt'))

    def test_deleteDirectory(self):
        os.mkdir('foo')
        deleteDirectory(None, 'foo')
        self.assertTrue(not os.path.exists('foo'))
        os.makedirs('bar/buz')
        deleteDirectory(None, 'bar/buz')
        self.assertTrue(not os.path.exists('bar/buz'))
        deleteDirectory(None, 'bar')
        self.assertTrue(not os.path.exists('bar'))

    def test_delete(self):
        open('foo.txt', 'wb').write('a')
        delete(None, 'foo.txt')
        self.assertTrue(not os.path.exists('foo.txt'))
        os.mkdir('bar')
        delete(None, 'bar')
        self.assertTrue(not os.path.exists('bar'))

    def test_readFile(self):
        open('foo.txt', 'wb').write('a')
        self.assertEquals('a', readFile(None, 'foo.txt'))
        deleteFile(None, 'foo.txt')
    
    def test_readDirectory(self):
        os.makedirs('bar/buz/boo')
        open('bar/abc.txt', 'wb')
        open('bar/xyz.txt', 'wb')
        entries = readDirectory(None, 'bar')
        names = set([e.name for e in entries])
        self.assertEquals(set(['buz', 'abc.txt', 'xyz.txt']), set(names))
        os.remove('bar/abc.txt')
        os.remove('bar/xyz.txt')
        os.removedirs('bar/buz/boo')

    def test_writeFile(self):
        open('foo.txt', 'wb')
        writeFile(None, 'foo.txt', 'foo')
        self.assertEquals('foo', open('foo.txt').read())
        os.remove('foo.txt')

    def test_getMetadata_from_text_file(self):
        open('foo.txt', 'wb').write('abc')
        meta = getMetadata(None, 'foo.txt')
        self.assertTrue(isinstance(meta, FileMetadata))
        self.assertEquals(3, meta.contentLength)
        self.assertEquals(timestamp(os.stat('foo.txt').st_mtime), meta.lastModified)
        self.assertFalse(meta.executable)
        self.assertEquals('text/plain', meta.contentType)
        self.assertTrue(meta.isText)
        self.assertEquals('utf-8', meta.textEncoding)
        os.remove('foo.txt')

    def test_getMetadata_from_text_file_with_encoding_decl(self):
        open('foo.txt', 'wb').write('<?caty-encoding encoding="EUC-JP" ?>\nabc')
        meta = getMetadata(None, 'foo.txt')
        self.assertEquals('euc-jp', meta.textEncoding)
        os.remove('foo.txt')

        open('foo.txt', 'wb').write('<?xml version="1.0" ?>\nabc')
        meta = getMetadata(None, 'foo.txt')
        self.assertEquals('utf-8', meta.textEncoding)
        os.remove('foo.txt')

        open('foo.txt', 'wb').write('<?xml version="1.0" encoding="EUC-JP" ?>\nabc')
        meta = getMetadata(None, 'foo.txt')
        self.assertEquals('euc-jp', meta.textEncoding)
        os.remove('foo.txt')

    def test_getMetadata_from_directory(self):
        os.makedirs('bar/buz/boo')
        open('bar/abc.txt', 'wb')
        open('bar/xyz.txt', 'wb')

        meta = getMetadata(None, 'bar')
        self.assertEquals(timestamp(os.stat('bar').st_mtime), meta.lastModified)

        os.remove('bar/abc.txt')
        os.remove('bar/xyz.txt')
        os.removedirs('bar/buz/boo')

if __name__ == '__main__': 
    result = unittest.TestResult()
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(StdFsTest))
    suite.run(result)
    if not result.wasSuccessful():
        print '--------'.join(('%s:\n%s' % (f[0]._testMethodName, f[1]) for f in result.failures + result.errors))
