
# coding: utf-8
import sys
import os
sys.path.insert(0, os.path.abspath('../../../'))
import unittest
from caty.mafs.authorization import *

PATH = [
    '/foo',
    '/public/foo',
    '/share/foo',
    '/admin/foo',
]

def dummy(token, path, *args, **kwds):pass

class SimpleCheckerTest(unittest.TestCase):
    def failIfSucc(f, *args):
        try:
            f(*args)
            self.fail()
        except:
            pass

    def testAdmin(self):
        u"""admin はあらゆる操作が可能。
        """
        token = AuthoriToken('admin')
        for path in PATH:
            simple_checker(CREATE)(dummy)(token, path)
            simple_checker(READ)(dummy)(token, path)
            simple_checker(UPDATE)(dummy)(token, path)
            simple_checker(DELETE)(dummy)(token, path)

    def testMemberCerate(self):
        u"""member は /share 以下のみファイル作成可能
        """
        token = AuthoriToken('member')
        simple_checker(CREATE)(dummy)(token, '/share/foo')
        self.failIfSucc(simple_checker(CREATE)(dummy), token, '/foo')
        self.failIfSucc(simple_checker(CREATE)(dummy), token, '/public/foo')
        self.failIfSucc(simple_checker(CREATE)(dummy), token, '/admin/foo')

    def testMemberRead(self):
        u"""member は /admin 以下でないファイルを読める
        """
        token = AuthoriToken('member')
        simple_checker(READ)(dummy)(token, '/share/foo')
        simple_checker(READ)(dummy)(token, '/foo')
        simple_checker(READ)(dummy)(token, '/public/foo')
        self.failIfSucc(simple_checker(READ)(dummy), token, '/admin/foo')

    def testMemberUpdate(self):
        u"""creaet と同じ
        """
        token = AuthoriToken('member')
        simple_checker(UPDATE)(dummy)(token, '/share/foo')
        self.failIfSucc(simple_checker(UPDATE)(dummy), token, '/foo')
        self.failIfSucc(simple_checker(UPDATE)(dummy), token, '/public/foo')
        self.failIfSucc(simple_checker(UPDATE)(dummy), token, '/admin/foo')

    def testMemberDelete(self):
        u"""削除は抑制しておく
        """
        token = AuthoriToken('member')
        for path in PATH:
            self.failIfSucc(simple_checker(DELETE)(dummy), token, path)

    def testGuestRead(self):
        u"""guest は /public と /share 以下のファイルだけを読み取れる
        """
        token = AuthoriToken('guest')
        simple_checker(READ)(dummy)(token, '/share/foo')
        self.failIfSucc(simple_checker(READ)(dummy), token, '/foo')
        simple_checker(READ)(dummy)(token, '/public/foo')
        self.failIfSucc(simple_checker(READ)(dummy), token, '/admin/foo')

    def testGuestModify(self):
        u"""guest は削除も作成も更新もできない
        """
        token = AuthoriToken('guest')
        for path in PATH:
            self.failIfSucc(simple_checker(CREATE)(dummy), token, path)
            self.failIfSucc(simple_checker(UPDATE)(dummy), token, path)
            self.failIfSucc(simple_checker(DELETE)(dummy), token, path)

if __name__ == '__main__': 
    result = unittest.TestResult()
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(SimpleCheckerTest))
    suite.run(result)
    if not result.wasSuccessful():
        print '--------'.join(('%s:\n%s' % (f[0]._testMethodName, f[1]) for f in result.failures + result.errors))

