# coding: utf-8
import sys
import os
sys.path.insert(0, os.path.abspath('../../../'))
import unittest
from caty.mafs.builder import *
from caty.mafs.authorization import AuthoriToken

def mafs():
    return build_mafs_module(root_dir=os.getcwd())

class MafsBuilderTest(unittest.TestCase):
    def testHasEnoughFunction(self):
        mod = mafs()
        self.assertTrue(hasattr(mod, 'createFile'))
        self.assertTrue(hasattr(mod, 'putFile'))
        self.assertTrue(hasattr(mod, 'createDirectory'))
        self.assertTrue(hasattr(mod, 'readFile'))
        self.assertTrue(hasattr(mod, 'readDirectory'))
        self.assertTrue(hasattr(mod, 'delete'))
        self.assertTrue(hasattr(mod, 'deleteFile'))
        self.assertTrue(hasattr(mod, 'deleteDirectory'))
        self.assertTrue(hasattr(mod, 'writeFile'))
        self.assertTrue(hasattr(mod, 'getMetadata'))

    def test_createFile(self):
        mod = mafs()
        try:
            mod.createFile(AuthoriToken('admin'), '/foo.txt')
            self.assertTrue(os.path.exists('foo.txt'))
        finally:
            os.remove('foo.txt')

    def test_getMetadata(self):
        mod = mafs()
        open('foo.txt', 'wb').write('<?caty-encoding encoding="sjis" ?>')
        try:
            metadata = mod.getMetadata(AuthoriToken('admin'), '/foo.txt')
            self.assertTrue(metadata.isText)
            self.assertTrue(metadata.textEncoding == 'sjis')
        finally:
            os.remove('foo.txt')

from fileobject import FileObjectTest, DirectoryObjectTest
class FileObjectBuilderTest(FileObjectTest):
    def create_fo(self, path, mode='rb'):
        token = AuthoriToken('admin')
        m = build_mafs_module(root_dir='', path_checker=lambda s: ())
        fom = build_fo_module(m)
        return fom.FileObject(token, path, mode)

class DirectoryObjectBuilderTest(DirectoryObjectTest):
    def create_do(self, path):
        token = AuthoriToken('admin')
        m = build_mafs_module(root_dir='', path_checker=lambda s: ())
        fom = build_fo_module(m)
        return fom.DirectoryObject(token, path)

if __name__ == '__main__': 
    result = unittest.TestResult()
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(MafsBuilderTest))
    suite.addTest(unittest.makeSuite(FileObjectBuilderTest))
    suite.addTest(unittest.makeSuite(DirectoryObjectBuilderTest))
    suite.run(result)
    if not result.wasSuccessful():
        print '--------'.join(('%s:\n%s' % (f[0]._testMethodName, f[1]) for f in result.failures + result.errors))

