# coding: utf-8
from __future__ import with_statement
import sys
import os
from datetime import datetime
import time
sys.path.insert(0, os.path.abspath('../../../'))
import unittest
from caty.mafs.fileobject import *
from caty.mafs.authorization import *
from caty.mafs import stdfs

class FileObjectTest(unittest.TestCase):
    def create_fo(self, path, mode='rb'):
        return FileObject(AuthoriToken('admin'), path, mode, stdfs)

    def testCreate(self):
        f = self.create_fo('foo.txt', 'wb')
        try:
            self.assertTrue(os.path.exists('foo.txt'))
        finally:
            os.remove('foo.txt')

    def testWrite(self):
        try:
            with self.create_fo('foo.txt', 'wb') as f:
                f.write('a')
                f.write('b')
            self.assertEquals(open('foo.txt').read(), 'ab')
        except Exception, e:
            print e
            raise 
        finally:
            os.remove('foo.txt')

    def testRead(self):
        open('foo.txt', 'wb').write('abc\ndef')
        try:
            f = self.create_fo('foo.txt')
            self.assertEquals('abc\ndef', f.read())
        finally:
            os.remove('foo.txt')

    def testRemoveCatyEncoding(self):
        open('foo.txt', 'wb').write('<?caty-encoding encoding="sjis" ?>\nabc\ndef')
        try:
            f = self.create_fo('foo.txt')
            self.assertEquals('abc\ndef', f.read())
            self.assertEquals('sjis', f.encoding)
        finally:
            os.remove('foo.txt')

    def testDelete(self):
        open('foo.txt', 'wb')
        try:
            f = self.create_fo('foo.txt')
            f.delete(None)
            self.assertFalse(os.path.exists('foo.txt'))
        finally:
            if os.path.exists('foo.txt'):
                os.remove('foo.txt')

    def testIsModifiedSince(self):
        open('foo.txt', 'wb')
        try:
            f = self.create_fo('foo.txt')
            self.assertTrue(f.is_modifed_since(100.0))
            self.assertFalse(f.is_modifed_since(time.time() + 1000.0))
        finally:
            os.remove('foo.txt')

    def testGetMetadata(self):
        open('foo.txt', 'wb').write('abc')
        try:
            f = self.create_fo('foo.txt')
            self.assertTrue(isinstance(f.last_modified, datetime))
            self.assertFalse(f.is_hidden)
            self.assertFalse(f.is_executable)
            self.assertEquals('utf-8', f.encoding)
            self.assertEquals(3, f.content_length)
            self.assertEquals('text/plain', f.content_type)
            self.assertTrue(f.is_text)
        finally:
            os.remove('foo.txt')

class DirectoryObjectTest(unittest.TestCase):
    def create_do(self, path):
        return DirectoryObject(AuthoriToken('admin'), path, stdfs)

    def testCreate(self):
        d = self.create_do('bar')
        try:
            d.create()
            self.assertTrue(os.path.exists('bar'))
        finally:
            os.rmdir('bar')

    def testRead(self):
        os.makedirs('bar/buz/boo')
        open('bar/foo.txt', 'wb')
        open('bar/buz/foo.txt', 'wb')
        open('bar/buz/boo/foo.txt', 'wb')
        try:
            d = self.create_do('bar')
            entries = list(d.read())
            self.assertEquals(2, len(entries))
            entries = list(d.read(True))
            self.assertEquals(3, len(entries))
        finally:
            os.remove('bar/buz/foo.txt')
            os.remove('bar/buz/boo/foo.txt')
            os.remove('bar/foo.txt')
            os.rmdir('bar/buz/boo')
            os.rmdir('bar/buz')
            os.rmdir('bar')

if __name__ == '__main__': 
    result = unittest.TestResult()
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(FileObjectTest))
    suite.addTest(unittest.makeSuite(DirectoryObjectTest))
    suite.run(result)
    if not result.wasSuccessful():
        print '--------'.join(('%s:\n%s' % (f[0]._testMethodName, f[1]) for f in result.failures + result.errors))

