#!/usr/bin/env python
#coding:utf-8
import cmd, glob, os
from caty.shell.system import setup_caty, Environment, StdStream
from caty.shell.script.builtin import Print
from caty.shell.script.builder import CommandCombinator
from caty.shell.script.parser import NothingTodo
from caty.jsontools import pp
from caty.mafs.authorization import AuthoriToken

class CatyShell(cmd.Cmd):
    def __init__(self, command_builder_class, fs, env, stream, dir=os.getcwd()):
        cmd.Cmd.__init__(self)
        self.prompt = 'caty> '
        self.intro = """Caty interactive shell
        """
        facilities = {
            'mafs': fs,
            'env': env,
            'stream': stream,
            'config': None,
            'var': None
        }
        builder = command_builder_class(facilities)
        self.command_builder  = builder
        self.env = env
        self.stream = stream
        self.last_result = 0

    def default(self, line):
        if line == 'EOF' or line.strip() in ('quit', 'exit'):
            return True
        else:
            if line.strip() in ('\n', '\r', '\r\n'):
                # 単なる空行には何もしない
                return False
            try:
                # 一連のパイプラインが入力しきっていない場合、 None が返る
                try:
                    c = self.command_builder.build(line.strip())
                except NothingTodo:
                    self.prompt = 'caty> '
                    return False
                if c:
                    self.last_result = c()
                    o = pp(c.output)
                    self.stream.write(o + '\n')
                    self.prompt = 'caty> '
                else:
                    self.prompt = '> '
            except Exception, e:
                import traceback
                print traceback.format_exc()
                msg = unicode(e.message, self.stream.encoding)
                print msg.encode(self.stream.encoding)
                self.prompt = 'caty> '
            finally:
                self.env.request.clear()

    def format(self, obj):
        if isinstance(obj, unicode):
            o = '"%s"' % obj
        elif isinstance(obj, dict):
            o = pp(obj)
        elif isinstance(obj, list):
            o = '[%s]' % ', '.join([self.format(v) for v in obj])
        else:
            o = str(obj)
        return o

    def completedefault(self, text, line, bi, ei):
        files = glob.glob(text + '*')
        return list(files)

    def emptyline(self):
        return False

import sys
try:
    import simplejson as json
except ImportError:
    import json
def main():
    if not os.path.exists('config.json'):
        print 'There is no cofinguration file.'
        print 'Created config.json.'
        open('config.json', 'wb').write("""
{
    "debug": true,
    "secret_key": "YouCanSetAnyStringHere",
    "mafs" : {
        "filesystem" : "caty.mafs.stdfs",
        "root_directory" : %s,
        "mime_types": {
            ".entry": "text/plain",
            ".wiki": "text/plain",
            ".jpg": "image/jpeg",
            ".gif": "image/gif",
            ".png": "image/png",
            ".csv": "text/plain"
        }
    },
    
    "web" : {
        "port": 8000,
        "session_timeout": 3600,
        "server_module": "caty.web.simple.app",
        "default_assoc": {
            ".html": "print -e %%1",
            ".htm": "print -e %%1",
            ".xhtml": "print -e %%1",
            ".xhtm": "print -e %%1",
            "default": "print -r %%1"
        },

        "login_path": "/login.html",
        "server_url": "http://localhost:8000",
        "indexfiles": ["index.html"],
        "script_ext" : [".cgi"],

        "encoding" : "utf-8"
     }
}
""" % json.dumps(os.getcwd().replace('python', 'webapp')))
    setup_caty('config.json', True)
    from caty.shell.script import ShellCommandFacade
    from caty.shell.session import Session
    from caty.mafs import filesystem as fs
    if len(sys.argv) > 1:
        arg = sys.argv[1]
        if (len(arg) == 0 or arg.isspace()):
            return 0
        else:
            return build_commands(arg)()
    else:
        if sys.platform == 'win32':
            import pyreadline as readline
        else:
            import readline
        orgdelims = readline.get_completer_delims()
        newdelims = orgdelims.replace('/', '')
        readline.set_completer_delims(newdelims)
        env = Environment(Session('session_scope'), Session('request_scope'))
        CatyShell(ShellCommandFacade, fs.bind_token(AuthoriToken('admin')), env, StdStream()).cmdloop()
        return 0

if __name__ == '__main__':
    sys.exit(main())

