# coding:utf-8
from caty.shell.script.parser import ScriptParser, NothingTodo
from caty.shell.script.builder import CommandBuilder, CommandCombinator
from caty.shell.script import node
from caty.shell.script import builtin
from caty.shell.script.exception import ScriptError
from caty.shell.script.cmdbase import Builtin, Command, PipelineInterruption, PipelineErrorExit, compile_builtin
from caty.util import memoize

import types

search_path = {}

@memoize
def initialize(registrar):
    compile_builtin(builtin, registrar)
    compile_builtin(node, registrar)
    for n, profile_container in registrar.command_profiles:
        try:
            cmd = profile_container.get_command_class()
            cmd.profile_container = profile_container
            search_path[n] = cmd
        except Exception, e:
            print e

class AbstractCommandFacade(object):
    u"""コマンド文字列からコマンドを構築するためのオブジェクト。
    実際の処理は CommandBuilder や parse_command に移譲し、
    このクラスでは対話シェルからの入力の受付や前二者の紐付けなどを行う。
    """
    def __init__(self, facility):
        self.builder = CommandBuilder(facility, search_path)
        self.prev_string = u''
    
    def build(self, string, *parameters):
        u"""コマンド文字列とコマンドライン引数リストを受け取り、
        コマンドオブジェクトを構築する。
        実際の構築処理はサブクラスの _build で行い、
        このメソッドでは引数の展開のみを行う。
        """
        for i, v in enumerate(parameters):
            k = u'%%%d' % (i+1)
            string = string.replace(k, v)
        return self._build(string)

    def _build(self, string):
        raise NotImplemtentedError
    

class ShellCommandFacade(AbstractCommandFacade):
    def _build(self, string):
        u"""コマンド文字列を受け取り、実行可能なパイプラインを構築する。
        対話シェルから使われることを想定しているので、入力途中で改行されたりなど
        パイプラインが構築し切れていない場合は以前の状態を保存し、
        継続行の入力も受け付けるものとする。
        """
        from topdown import ParseFailed
        string = self.prev_string + '\n' + string if self.prev_string else string
        parser = ScriptParser(string, self.builder)
        try:
            cmd = parser.parse()
        except ParseFailed, e:
            self.prev_string = u''
            raise
        if not cmd:
            self.prev_string = string
            return None
        else:
            self.prev_string = u''
            return apply_builder(self, cmd)

class ScriptCommandFacade(AbstractCommandFacade):
    def _build(self, string):
        u"""コマンド文字列を受け取り、実行可能なパイプラインを構築する。
        呼び出し可能な状態のオブジェクトが構築できなかった場合、即座にエラーとする。
        """
        parser = ScriptParser(string, self.builder)
        cmd = parser.parse()
        if not cmd:
            raise ScriptError('Syntax Error: %s' % string)
        else:
            return apply_builder(self, cmd)

def apply_builder(builder, cmd):
    u"""print コマンドは内部的にアソシエーションを解決する必要があるので、
    実行前に builder （実体は CommandFacade）を注入する。
    初期化時に行わないのは、オブジェクトの deepcopy 処理での不具合を回避するため。
    """
    if isinstance(cmd, builtin.Print):
        cmd.builder = builder
    elif isinstance(cmd, CommandCombinator):
        apply_builder(builder, cmd.bf)
        apply_builder(builder, cmd.af)
    return cmd


