# coding:utf-8
from caty.shell.script.cmdbase import Builtin, Command

class CommandBuilder(object):
    def __init__(self, facilities, search_path):
        self.facilities = facilities
        self.search_path = search_path

    def build(self, chunk):
        u"""コマンド文字のチャンクをコマンド名と引数のリストに分割し、呼び出し可能なコマンドオブジェクトを返す。
        """
        chunk = list(chunk)
        name = chunk.pop(0)
        cls = self.search_path[name]
        args = chunk
        cmd = cls(args)
        cmd.set_facility(self.facilities)
        return cmd


class CommandCombinator(Command):
    u"""二つのコマンドを受け取り、それらを順次実行するコマンドのラッパークラス。
    """
    def __init__(self, bf, af):
        self.bf = bf
        self.af = af
    
    def convert(self, value):
        return self.bf.convert(value)

    def set_facility(self, facilities):
        self.bf.set_facility(facilities)
        self.af.set_facility(facilities)

    @property
    def in_schema(self):
        return self.bf.in_schema

    @property
    def out_schema(self):
        return self.af.out_schema

    def input():
        def _get(self):
            return self.bf.input
        def _set(self, value):
            self.bf.input = value
        return _get, _set
    input = property(*input())

    def output():
        def _get(self):
            return self.af.output
        def _set(self, value):
            self.af.output = value
        return _get, _set
    output = property(*output())

    def __call__(self):
        r = self.bf()
        out = self.bf.output
        self.af.input = out
        return self.af()

def combine(chunks):
    return reduce(CommandCombinator, chunks)

