# coding: utf-8
from HTMLParser import HTMLParser
import shlex
class CatyShebangError(Exception): pass

def parse_meta(s):
    d = {}
    for t in shlex.split(str(s)):
        try:
            k, v = t.split('=', 1)
        except:
            raise CatyShebangError(s)
        else:
            d[k.strip().encode()] = v.strip().encode()
    return d

class CatyShebangParser(HTMLParser):
    u"""shebang の読み取りを行う。
    shebang 以外の部分はそのまま書き出すが、
    shebang は除去される。
    """
    def __init__(self, associate):
        HTMLParser.__init__(self)
        self._buffer = []
        self._meta = {}
        self._associate = associate
        self._script = ''

    content = property(lambda self: ''.join(self._buffer))
    meta = property(lambda self: self._meta)
    script = property(lambda self: self._script.strip())

    def handle_starttag(self, tag, attrs):
        a = ' '.join(['%s="%s"' % attr for attr in attrs])
        if a:
            self._buffer.append('<%s %s>' % (tag, a))
        else:
            self._buffer.append('<%s>' % (tag))

    def handle_startendtag(self, tag, attrs):
        a = ' '.join(['%s="%s"' % attr for attr in attrs])
        if a:
            self._buffer.append('<%s %s/>' % (tag, a))
        else:
            self._buffer.append('<%s/>' % (tag))

    def handle_endtag(self, tag):
        self._buffer.append('</%s>' % tag)

    def handle_data(self, data):
        self._buffer.append(data)

    def handle_charref(self, name):
        self._buffer.append('&%s;' % name)

    def handle_entityref(self, name):
        self._buffer.append('&%s;' % name)

    def handle_comment(self, data):
        self._buffer.append('<!-- %s -->' % data)

    def handle_pi(self, data):
        data = data[:-1] # ? が末尾に残ってしまうので除去
        if data.startswith('caty-meta') and not self._meta:
            self._meta = parse_meta(data.replace('caty-meta', ''))
        elif data.startswith('caty-script') and self._associate and not self._script:
            self._script = data.replace('caty-script', '').strip()
        else:
            self._buffer.append('<?%s?>' % data)

def parse(s, associate=False):
    parser = CatyShebangParser(associate)
    parser.feed(s)
    return parser.meta, parser.script, parser.content

