#coding:utf-8
from __future__ import with_statement
u"""ユーザ管理モジュール。
Caty においてユーザ情報はすべて /admin/users.json ファイルに格納される。
ユーザ情報もまた JSON 形式となっており、そのフォーマットは以下のとおりである。

object {
    "id" : string,
    "password": string
}

この形式は将来において変更される可能性がある。
"""

from hashlib import sha1
import sys
from time import time
from threading import RLock
import random
import caty.jsontools as json

def _get_fs():
    from caty.mafs import filesystem, AuthoriToken
    return filesystem.bind_token(AuthoriToken('admin'))

def generate_password(s):
    from caty.shell.system import SECRET_KEY
    salt = sha1(SECRET_KEY + str(int(time()) + random.randint(0, sys.maxint - 1))).hexdigest()
    password = sha1(salt + s).hexdigest()
    return salt + password

def authenticate(incoming, password):
    salt = password[0:40]
    p = password[40:]
    a = sha1(salt + incoming).hexdigest()
    return a == p

def login(username, password):
    fs = _get_fs()
    userdb = json.load(fs.open('/admin/users.json'))
    u = userdb[username]
    if authenticate(password, u['password']):
        return True, u['grant']
    else:
        return False, ''

def add_user(username, grant, password):
    fs = _get_fs()
    password = generate_password(password)
    with RLock():
        if fs.FileObject('/admin/users.json', 'rb').exists:
            userdb = json.load(fs.open('/admin/users.json'))
        else:
            userdb = {}
        userdb[username] = {
            'grant': grant,
            'password': password
            }
        json.dump(userdb, fs.open('/admin/users.json', 'wb'))
    return

