# coding: utf-8
class ICompiler(object):
    u"""バイトコードコンパイラインターフェース。
    ファイル類似オブジェクトからバイトコードを生成するためのインタフェースである。
    """
    def compile(self, fo):
        raise NotImplementedError


class Compiler(ICompiler):
    u"""バイトコードコンパイラ。
    実際には構文解析が終了した時点でバイトコンパイルされているので、
    このクラスは字句解析と構文解析をまとめるラッパーとして作用する。
    """
    def __init__(self, lexer, parser):
        self._lexer_class = lexer
        self._parser_class = parser
    
    def compile(self, fo):
        u"""ファイル類似オブジェクトかパス名を引数とし、バイトコードの不変リストを返す。
        """
        lexer = self._lexer_class(fo)
        parser = self._parser_class(lexer)
        parser.parse_template()
        return parser.generate()

