header {
import sys
from StringIO import StringIO
import SmartyExprLexer
import caty.template.core.instructions as ins
from caty.template.core.vm import Bytecode, Stack, TemplateRuntimeError
}

header "SmartyParser.__init__" {
    self.used_label = set()
    self.if_label_stack = Stack()
    self.upcoming_end_stack = Stack()
    self.for_label_stack = Stack()
    self.codes = [] # list of tuple(INSTRUCTION, VALUE)
}

options {
    language = "Python";
}


class SmartyParser extends Parser;
    {
        def generate(self):
            def _():
                for x in self.codes:
                    yield Bytecode(x[0], x[1])
            return tuple(list(_()))

        def createLoadingVariableCode(self, e):
            filter = None
            argc = 0
            for token in SmartyExprLexer.Lexer(StringIO(e.getText())):
                t = token.getType()
                s = token.getText()
                if t == SmartyExprLexer.VARIABLE:
                    varname = s.replace("->", ".")
                    b = (ins.LOAD, varname)
                    self.codes.append(b)
                elif t == SmartyExprLexer.FILTER:
                    if filter:
                        b = (ins.CALL, (filter, argc))
                        self.codes.append(b)
                        argc = 0
                    filter = s
                elif t == SmartyExprLexer.FILTER_ARGS:
                    b = (ins.PUSH, s)
                    self.codes.append(b)
                    argc += 1
                elif t == SmartyExprLexer.INDEX:
                    self.codes.append((ins.PUSH, int(s)))
                    self.codes.append((ins.AT, None))
            if filter:
                b = (ins.CALL, filter, argc)
                self.codes.append(b)

        def createIfCode(self, i):
            self.createLoadingVariableCode(i)
            v = i.getText()
            label = str(id(i))
            while label in self.used_label:
                label = str(id(label))
            self.used_label.add(label)
            self.if_label_stack.push(label)
            if v.startswith('!'):
                self.code.append((INS.NOT, None))
                b = (ins.JMPUNLESS, label)
            else:
                b = (ins.JMPUNLESS, label)
            self.codes.append(b)
         


    }
    tokenDefinitions:
    IF
    IFNOT
    ELSEIF
    ELSE
    ENDIF
    FOREACH
    ENDFOR
    INCLUDE
    ;
    parse_template:(
        e:EXPR {
            self.createLoadingVariableCode(e)
            b = (ins.POP, None)
            self.codes.append(b)
        }
        | i:IF {
            self.createIfCode(i)
            self.upcoming_end_stack.push(self.if_label_stack.top() + "end")
        }
        | ei:ELSEIF {
            self.codes.append((ins.JMP, self.upcoming_end_stack.top()))
            self.codes.append((ins.LABEL, self.if_label_stack.pop()))
            self.createIfCode(ei)
        }
        | el:ELSE {
            self.codes.append((ins.JMP, self.upcoming_end_stack.top()))
            self.codes.append((ins.LABEL, self.if_label_stack.pop()))
        }
        | en:ENDIF {
            if not self.if_label_stack.is_empty():
                self.codes.append((ins.LABEL, self.if_label_stack.pop()))
            if not self.upcoming_end_stack.is_empty():
                self.codes.append((ins.LABEL, self.upcoming_end_stack.pop()))
        }
        | fo:FOREACH {
            label = str(id(fo))
            while label in self.used_label:
                label = str(id(label))
            self.used_label.add(label)
            loop_var = ""
            loop_item = ""
            loop_key = ""
            loop_name = ""
            for token in SmartyExprLexer.Lexer(StringIO(fo.getText())):
                t = token.getType()
                s = token.getText()
                if t == SmartyExprLexer.LOOP_VAR:
                    loop_var = s
                elif t == SmartyExprLexer.LOOP_KEY:
                    loop_key = s
                elif t == SmartyExprLexer.LOOP_ITEM:
                    loop_item = s
                elif t == SmartyExprLexer.LOOP_NAME:
                    loop_name = s
            if not loop_var or not loop_item:
                raise TemplateRuntimeError("Syntax error: %s" % fo.getText())
            if not loop_name:
                loop_name = "default"
            index_name = "smarty.foreach.%s.index" % loop_name
            iteration_name = "smarty.foreach.%s.iteration" % loop_name
            counter_name = "smarty.foreach.%s.total" % loop_name
            first = "smarty.foreach.%s.first" % loop_name
            last = "smarty.foreach.%s.last" % loop_name
            self.codes.append((ins.NEWCTX, None))
            self.codes.append((ins.PUSH, -1))
            self.codes.append((ins.CPUSH, index_name))
            self.codes.append((ins.PUSH, 0))
            self.codes.append((ins.CPUSH, iteration_name))
            self.codes.append((ins.LEN, loop_var))
            self.codes.append((ins.CPUSH, counter_name))

            self.codes.append((ins.LABEL, label))

            self.codes.append((ins.LOAD, index_name))
            self.codes.append((ins.PUSH, 1))
            self.codes.append((ins.ADD, None))
            self.codes.append((ins.CPUSH, index_name))

            self.codes.append((ins.LOAD, iteration_name))
            self.codes.append((ins.PUSH, 1))
            self.codes.append((ins.ADD, None))
            self.codes.append((ins.CPUSH, iteration_name))
        
            self.codes.append((ins.PUSH, 0))
            self.codes.append((ins.LOAD, index_name))
            self.codes.append((ins.EQ, None))
            self.codes.append((ins.CPUSH, first))

            self.codes.append((ins.LOAD, counter_name))
            self.codes.append((ins.LOAD, iteration_name))
            self.codes.append((ins.EQ, None))
            self.codes.append((ins.CPUSH, last))

            self.codes.append((ins.LOAD, loop_var))
            self.codes.append((ins.LOAD, index_name))
            self.codes.append((ins.AT, None))
            self.codes.append((ins.CPUSH, loop_item))

            self.for_label_stack.push((label, last))

        }
        | ef:ENDFOR {
            label, last = self.for_label_stack.pop()
            self.codes.append((ins.LOAD, last))
            self.codes.append((ins.JMPUNLESS, label))
            self.codes.append((ins.DELCTX, None))
        }
        | l:LITERAL {
            st = l.getText()
            b = (ins.STRING, st)
            self.codes.append(b)
        }
        | inc:INCLUDE {
            for token in SmartyExprLexer.Lexer(StringIO(inc.getText())):
                b = (ins.INCLUDE, token.getText())
                self.codes.append(b)
        }
    )*
    ;

class SmartyLexer extends Lexer;

options {
    k=9;
    filter=true;
    charVocabulary = '\u0000'..'\uFFFE';
}
LITERAL : ('\\'! '{' //Escaped delimiter
        | '\\' ~'{'
        | ~'{'
        )+
        {
            $getText;
        }
        ;

ACTION :
      COMMENT { $skip; }
    | '{'! (' '!)* EXPR (' '!)* '}'!  { $setType(EXPR);}
    | "{if"! (' '!)+ IF_EXPR (' '!)* '}'! { $setType(IF); }
    | "{elseif"! (' '!)+ IF_EXPR (' '!)* '}'! { $setType(ELSEIF); }
    | "{else}"! { $setType(ELSE); }
    | "{/if}"! { $setType(ENDIF); }
    | "{foreach"! (' '!)+ FOREACH_EXPR (' '!)* '}'! 
        { $setType(FOREACH);}
    | "{/foreach}"! { $setType(ENDFOR) }
    | "{include"! (' '!)+ INC_EXPR (' '!)* '}'!
        { $setType(INCLUDE);}
    ;

protected
NL
options { generateAmbigWarnings = false; }
    :   '\r'
    |   '\n'
    |   '\r' '\n'
    ;

protected
COMMENT   : "{*" 
            ( options {greedy = false;} 
            : NL
            | .
            ) *
            "*}"
        ;
protected
IF_EXPR : ('!')? EXPR
        ;
protected
EXPR : VARIABLE (FILTER)*
        ;
protected
FILTER : '|' ('a'..'z'|'A'..'Z')('a'..'z'|'A'..'Z'|'0'..'9')*
          (FILTER_ARGS)*
        { $setType(FILTER); }
        ;
protected
FILTER_ARGS : ':' (('"' (~'"') '"')|(('0'..'9')+))
        ;
protected
VARIABLE : '$' ('a'..'z'|'A'..'Z')('a'..'z'|'A'..'Z'|'0'..'9')*
        (
           ("." ('a'..'z'|'A'..'Z')('a'..'z'|'A'..'Z'|'0'..'9')*)
           |('[' ('0'..'9')+ ']')
        ) *
        { $setType(VARIABLE); }
        ;
protected
FOREACH_EXPR :
        ((LOOP_VAR)|(LOOP_ITEM)|(LOOP_NAME))
        ((' ') ((LOOP_VAR)|(LOOP_ITEM)|(LOOP_NAME)))*
        ;

private
LOOP_VAR : "from" (' ')? '=' (' ')? ('$') (SYMBOL)
        ;
private
LOOP_ITEM : "item" (' ')? '=' (' ')? (SYMBOL)
        ;
/*private
LOOP_KEY : "key" (' ')? '=' (' ')? (SYMBOL)
        ;*/
private
LOOP_NAME : "name" (' ')? '=' (' ')? (SYMBOL)
        ;
protected
INC_EXPR : "file" (' ')? '=' (' ')? '"' (~'"')+ '"'
        ;
protected
NO_ESC : (~'\\')? ;
protected
SYMBOL :  ('a'..'z'|'A'..'Z')('a'..'z'|'A'..'Z'|'0'..'9'|'.')*;

