#coding: utf-8
from __future__ import with_statement
from threading import RLock
u"""共通的に使われるユーティリティ関数群。
"""
class memoize(object):
    u"""関数にキャッシュ機能をつける。
    """
    def __init__(self, f):
        self.__function = f
        self.__cache = {}


    def __call__(self, *args):
        f = self.__function
        cache = self.__cache
        with RLock():
            if args in cache:
                return cache[args]
            else:
                cache[args] = f(*args)
                return cache[args]
    
    def clear(self):
        u"""キャッシュのクリアを行う。
        """
        self.__cache = {}

def merge_dict(a, b, conflict_mode='post'):
    u"""再帰的に辞書をマージしていく。
    conflict_mode は a, b でキーが重複する葉に対しての動作である。
    デフォルトでは 'post' が使われる。

    'post': b の値を使う
    'pre': a の値を使う
    'error': 例外を送出する

    関数オブジェクトを conflict_mode に渡すことも可能である。
    その関数は a[k] と b[k] の二つの値を引数として受け取れなければならない。
    関数の戻り値が 0 なら a が、 1 なら b が使われ、
    それ以外が戻り値の場合はエラーとなる。
    """
    for k, v in b.iteritems():
        if k in a:
            if isinstance(v, dict) and isinstance(a[k], dict):
                merge_dict(a[k], v, conflict_mode)
            else:
                if conflict_mode == 'post':
                    a[k] = v
                elif conflict_mode == 'pre':
                    pass
                elif conflict_mode == 'error':
                    raise Exception('Conflict: %s' % str(k))
                elif callable(conflict_mode):
                    r = conflict_mode(a[k], v)
                    if r == 0:
                        pass
                    elif r == 1:
                        a[k] = v
                    else:
                        raise Exception('Conflict: %s' % str(k))
                else:
                    raise Exception('Unknown mode: %s' % str(conflict_mode))
        else:
            a[k] = v
    return a

def try_parse(type, s):
    u"""s を type に変換する。
    失敗時には None を返す。
    """
    try:
        return type(str(s))
    except:
        return None

def bind1st(f, a):
    def bound(b):
        return f(a, b)
    return bound

def bind2nd(f, b):
    def bound(a):
        return f(a, b)
    return bound

