# coding: utf-8
u"""主に開発/デバッグ目的の簡単な Web アプリケーション。
"""
from copy import deepcopy
from Cookie import SimpleCookie
import wsgiref
import cgi
import sys
from wsgiref.simple_server import make_server, WSGIRequestHandler

from caty.jsontools import jstypes
from caty.mafs import filesystem as fs
from caty.mafs import AuthoriToken
from caty.shell.session import Session, SessionStorage
from caty.shell.system import setup_caty, Environment, DEBUG
from caty.shell.script import ScriptCommandFacade, PipelineInterruption, PipelineErrorExit
from caty.web.util import *
from caty.web import config

class WebStream(object):
    def __init__(self, environ, config):
        self.input = self._create_input(environ)
        self.encoding = 'utf-8'

    def _create_input(self, environ):
        u"""クエリの処理。
        TODO: ファイルアップロードに対応すること
        """
        if environ['REQUEST_METHOD'] == 'POST':
            l = environ['CONTENT_LENGTH']
            input = environ['wsgi.input'].read(int(l))
            a = cgi.parse_qs(input)
            qs = environ['QUERY_STRING']
            b = cgi.parse_qs(qs)
            a.update(b)
            return a
        else:
            qs = environ['QUERY_STRING']
            return cgi.parse_qs(qs)

    def _to_unicode(self, d):
        r = {}
        for k, v in d.iteritems():
            r[k] = map(lambda a:unicode(a, 'utf-8'), v)
        return r

    def read(self):
        return self.input

class CatyApp(object):
    def __init__(self, facade_class, filesystem, webconfig):
        self.facade_class = facade_class
        self.fs = filesystem
        self.config = webconfig

    def main(self, environ):
        env, var = self.create_env(environ)
        fs = self.fs.bind_token(self.create_auth_token(env))
        path = unicode(environ['PATH_INFO'], 'utf-8')
        var.put('PATH', jstypes.string, path)
        path = '/public' + path
        if path.startswith('/admin/'):
            config = self.config['admin']
        else:
            config = merge_config(path, deepcopy(self.config))

        if path.endswith('/'):
            for idf in config['indexfiles']:
                if fs.open(path + idf).exists:
                    path += idf
                    break
        if is_hidden_path(path):
            return {'status': u'403 FORBIDDEN'}
        stream = WebStream(environ, config)
        facilities = {
            'mafs':self.fs.bind_token(self.create_auth_token(env)),
            'env': env,
            'stream': stream,
            'config': config,
            'var': var
        }
        builder = self.facade_class(facilities)
        input_object = stream.read()
        cmd = self.build_cmd(builder, config, path, fs)
        cmd.input = cmd.convert(input_object) if input_object else None
        cmd()
        return cmd.output
    
    def build_cmd(self, builder, config, path, fs):
        p = path.rsplit('.', 1)
        if len(p) != 2:
            cmds = config['default_assoc'].get('__no_ext__').get(path.split('/')[-1], None)
            if not cmds:
                cmds = config['default_assoc'].get('__no_ext__').get('default')
            cmd = builder.build(cmds)
        else:
            ext = '.' + p[1]
            if ext in config['script_ext']:
                cmds = fs.open(path).read()
            else:
                cmds = config['default_assoc'].get(ext, None) or config['default_assoc']['default']
            cmd = builder.build(cmds, path)
        return cmd

    def __call__(self, environ, start_response):
        status = '200 OK'
        headers = []
        try:
            json = self.main(environ)
            headers = list(self.create_header(json['header']))
        except PipelineInterruption, e:
            json = e.json_obj
            headers = list(self.create_header(json['header']))
        except PipelineErrorExit, e:
            json = {'status': u'500 Internal Server Error'}
        except IOError, e:
            json = {'status': u'404 Not Found'}

        if 'status' in json:
            status = str(json['status'])
        start_response(status, headers)
        """if DEBUG:
            info = ["%s: %s\n" % (key, value)
               for key, value in environ.iteritems()]
            print ''.join(info)
            for k, v in json.header.iteritems():
                print k, v"""
        if 'body' in json:
            b = json['body']
            if isinstance(b, unicode):
                return [b.encode('utf-8')]
            else:
                return [b]
        else:
            return []

    def create_header(self, h):
        for k, v in h.iteritems():
            if isinstance(v, unicode):
                yield k, v.encode('utf-8')
            else:
                yield k, str(v)

    def create_env(self, environ):
        session = None
        if 'HTTP_COOKIE' in environ:
            c = environ['HTTP_COOKIE']
            cookie = SimpleCookie()
            cookie.load(c)
            if 'sessionid' in cookie:
                session = SessionStorage().restore(cookie['sessionid'].value)
        if session == None:
            session = Session('session_scope')
        return session, Session('request_scope')

    def create_auth_token(self, env):
        if env.exists('grant'):
            return AuthoriToken(env.get('grant'))
        else:
            return AuthoriToken('guest')

class CatyServerHandler(WSGIRequestHandler):
    def __init__(self, *args, **kwds):
        WSGIRequestHandler(self, *args, **kwds)
        self.logger = sys.stderr
        
    def set_logger(self, logger):
        self.logger = logger

    def get_stderr(self):
        return self.logger

def main():
    caty_app = CatyApp(ScriptCommandFacade, fs, config)
    print "Serving on port %d..." % config['port']
    httpd = make_server('', config['port'], caty_app)
    httpd.serve_forever()

