# -*- coding: utf-8 -*-
from __future__ import with_statement
from caty.shell.script import Command
import random
import datetime
try:
    import simplejson as json
except ImportError:
    import json
import os

def namecmp(a, b):
    u"""FileObjectのソート用の比較関数。
    パス名を比較に用いる。
    """
    return cmp(a.path, b.path)

# JSONオブジェクトのプロパティ名にuを忘れるとどうなる？

def fo_to_json(fo):
    return {u'name': os.path.basename(fo.path), u'is_dir': fo.is_dir}

def has_ext(fo, ext):
    return fo.path.endswith(ext)

# dirに対するreadは、ジェネレータを返す。
# command lsdir [string, string?] :: 
#      void -> list[object{"name":string, "is_dir":boolean}]
class LsDir(Command):
    def execute(self, opts, path, ext=""):
        dirobj = self.mafs.DirectoryObject(path)
        files = list(sorted([ent for ent in dirobj.read() if has_ext(ent,ext)], cmp=namecmp))
        r = map(fo_to_json, files)
        self.output = r


# command zerofile [string] :: void -> string
class ZeroFile(Command):
    def execute(self, opts, path):
       fo = self.mafs.open(path, "rb") 
       # wモードだと即座に作られる、既存ファイルは書き込むまで切りつめされない
       if (fo.exists):
           fo.close()
           r = u"file already exists"
       else:
           fo.close()
           fo = self.mafs.open(path, "wb") # これだけで作られる。
           fo.close()
           r = u"zero-file created"
       self.output = r

# command exists [string] :: void -> boolean
class Exists(Command):
    def execute(self, opts, path):
       fo = self.mafs.open(path)
       r = fo.exists
       fo.close()
       self.output = r

# command sort :: list[any] -> list[any]
class Sort(Command):
    def execute(self, opts):
        r = self.input
        r.sort()
        self.output = r

# command concat :: list[string] -> string
class Concat(Command):
    def execute(self, opts):
        r = "" 
        # r = ''
        for s in self.input:
            r += s
        self.output = r

# command readJson [string] :: void -> any
class ReadJson(Command):
    def execute(self, opts, fname):
        with self.mafs.open(fname) as f:
            j = json.load(f, 'utf-8')
        self.output = j

# command hello :: void -> string
class Hello(Command):
    def execute(self, opts):
        self.output = 'hello'

# command helloJa :: void -> string
class HelloJa(Command):
    def execute(self, opts):
        self.output = u"こんにちは"


# command pass :: _X -> _X // まだ型変数は使えない
class PassData(Command):
    def execute(self, opts):
        self.output = self.input

# command randint [integer, integer] :: void -> integer
class Randint(Command):
    def execute(self, opts, min, max):
        n = random.randint(min, max)
        self.output = n

# command randchoice :: list[any] -> any
class RandChoice(Command):
    def execute(self, opts):
        m = len(self.input)
        n = random.randint(0, m - 1)
        self.output = self.input[n]

# command nth [integer] :: list[any] -> any
class Nth(Command):
    def execute(self, opts, n):
        x = self.input[n]
        self.output = x

# command getpv [string] :: object {*:any} -> any
class GetPV(Command):
    def execute(self, opts, name):
        x = self.input[name]
        self.output = x

# command toupper :: string -> string;
class ToUpper(Command):
    def execute(self, opts):
        uc = self.input.upper()
        self.output = uc

# command tolower :: string -> string;
class ToLower(Command):
    def execute(self, opts):
        lc = self.input.lower()
        self.output = lc

# command timedGreeting {"hour":integer} :: void -> 
#  object {"message":string, "hour":integer, "minute":integer}
class TimedGreeting(Command):
    def execute(self, opts):
        d = datetime.datetime.today()
        if opts.hour:
            h = opts.hour
        else:
            h = d.hour 
        m = d.minute
        message = "hello"
        if 5 <= h and h < 11 :
            message = u"おはようございます"
#            message = u"Good morning"
        elif 11 <= h and h < 18 :
            message = u"こんにちは"
#            message = u"Good afternoon"
        elif 18 <= h and h < 21 :
            message = u"こんばんは"
#            message = u"Good evening"
        else: # (21 <= h and h < 24) or (0 <= h and h < 5)
            message = u"おやすみなさい"
#            message = u"Good night"
        self.output = {'message': message, 'hour':h, 'minute':m}

# command readFile [string] :: void -> string
#    reads mafs
class ReadFile(Command):
    def execute(self, opts, path):
        data = self.mafs.open(path, "r").read()
        self.output = data
