#coding:utf-8
from caty.shell.script import Command
from caty.util.collection import SortedList
from caty.util.creole import creole2html
from caty.jsontools.model import tagged
from itertools import islice
import os

def makeentry(fo):
    u"""FileObjectからblogEntryを構築する。
    この関数で構築されたblogEntryデータにはprev, nextの情報が
    完全には構築されていないので、別の関数で補う事とする。
    """
    c = fo.read().strip()
    body, tagline = c.rsplit('\n', 1)
    if tagline.startswith('tags:'):
        tags = map(lambda s: s.strip(), tagline.split(':', 1)[1].split(','))
    else:
        body = c
        tags = []
    title, rest = body.split('\n', 1)
    return {
        'link': os.path.split(os.path.split(fo.path)[0])[1] + '/',
        'title': title.strip('=').strip(),
        'body': creole2html(body),
        'description': rest[:140],
        'lastmod': unicode(fo.last_modified),
        'tags': tags,
        'prev': {"title": u"", "link": u"", "exists":False},
        'next': {"title": u"", "link": u"", "exists":False},
    }

def namecmp(a, b):
    u"""FileObjectのソート用の比較関数。
    パス名を比較に用いる。
    """
    return cmp(a.path, b.path)

import caty.jsontools as json
class ViewBlogIndex(Command):
    def execute(self, opts, path, items):
        # 検索クエリやタグの絞り込みがない場合。
        if self.input == None or ('tag' not in self.input):
            dirobj = self.mafs.DirectoryObject(path)
            tag = None
        elif 'tag' in self.input:
            tag = self.input['tag']
        if self.input == None or ('of' not in self.input):
            start = 0
        else:
            start = self.input['of']
        if tag:
            tags = json.load(self.mafs.open('/share/blog/tags.json'))
        else:
            tags = None
        files = list(reversed(sorted((e for e in dirobj.read() if self.filter(e, tag, tags)), cmp=namecmp)))
        length = len(files)
        res = []
        for f in files[start:start+items]:
            path = f.path
            o = self.mafs.open(path + '/index.entry')
            res.append(makeentry(o))
        if start >= items:
            prev = start - items
        else:
            prev = None
        if length > start + items:
            next = start + items
        else:
            next = None
        prevquery = self.create_query(prev, tag)
        nextquery = self.create_query(next, tag)
        self.output = {
            "items": res,
            "prev": prevquery,
            "next": nextquery
        }
        return 0

    def create_query(self, num, tag):
        q = []
        if num != None:
            q.append('of=%d' % num)
        if tag != None:
            q.append('tag=%s' % tag)
        if q:
            return u'&'.join(q)
        else:
            return None
    
    def filter(self, e, tag, tags):
        if not e.is_dir: return False
        if tag == None: return True
        if e.path not in tags[tag]: return False
        return True

class ViewBlogEntry(Command):
    def execute(self, opts, path):
        fo = self.mafs.open(path)
        res = makeentry(fo)
        parent_dir = os.path.split(os.path.split(path)[0])[0]
        prev_path = self.calc_path(path, -1)
        next_path = self.calc_path(path, 1)
        if prev_path:
            res['prev'] = {
                'link': prev_path + '/',
                'title': self.read_title(os.path.join(parent_dir, prev_path)),
                'exists': True,
            }
        if next_path:
            res['next'] = {
                'link': next_path + '/',
                'title': self.read_title(os.path.join(parent_dir, next_path)),
                'exists': True,
            }
        self.output = res
        return 0
    
    def calc_path(self, path, num):
        d, p = os.path.split(os.path.split(path)[0])
        n = int(p, 16) + num
        if n < 0:
            return None
        else:
            newpath = '%04x' % n
            if self.mafs.DirectoryObject(os.path.join(d, newpath)).exists:
                return unicode(newpath)

    def read_title(self, path):
        fo = self.mafs.open(os.path.join(path, 'index.entry'))
        title, rest = fo.read().split('\n', 1)
        return title.strip('=').strip()

class EditBlogEntry(Command):
    def execute(self, opts, path):
        if self.input != None:
            page = self.input['page'].strip('/')
            content = self.mafs.open(os.path.join(path, page, 'index.entry')).read()
        else:
            page = self.next_page(path)
            content = u''
        self.output = {
            'page': page,
            'content': content
        }
        return 0

    def next_page(self, path):
        dirobj = self.mafs.DirectoryObject(path)
        files = list(reversed(sorted((e for e in dirobj.read() if e.is_dir), cmp=namecmp)))
        return u'%04x' % (int(os.path.basename(files[0].path), 16) + 1)
        

class PostBlogEntry(Command):
    def execute(self, opts, path, app_path):
        page = self.input['page']
        content = self.input['content']
        if not self.mafs.DirectoryObject('%s/%s' % (path, page)).exists:
            self.mafs.DirectoryObject('%s/%s' % (path, page)).create()
        f = self.mafs.open('%s/%s/index.entry' % (path, page), 'wb')
        f.write(content.encode('utf-8'))
        f.close()
        self.output = {
            'status': u'302 Found',
            'header': {
                'Location': self.config['server_url'] + ('%s/%s/' % (app_path, page))
            },
        }
        return 0


