# coding:utf-8
from caty.shell.script import Command
from caty.util.collection import SortedList
from caty.util.creole import creole2html
from caty.jsontools.model import tagged
from itertools import islice
import os
from weblog import *

def to_wiki_title(path):
    return {'title': path.replace('.wiki', ''), 'file':path}

class WikiTitles(Command):
    def execute(self, opts, path):
        files = self.read_wiki_files(path)
        self.output = {'titles': map(to_wiki_title, map(lambda e: e.path.replace(path, '')[1:], files))}
        return 0

    def read_wiki_files(self, path):
        for e in self.mafs.DirectoryObject(path).read(True):
            if e.path.endswith('.wiki'):
                yield e

class RecentWiki(Command):
    def execute(self, opts, path, num=0):
        files = self.recent_wiki_files(path, num)
        self.output = {'pages': map(to_wiki_title, map(lambda e: e.path.replace(path, '')[1:], files))}
        return 0

    def recent_wiki_files(self, path, num):
        res = SortedList(lambda e1, e2: cmp(e2.last_modified, e1.last_modified))
        for e in self.mafs.DirectoryObject(path).read(True):
            if e.path.endswith('.wiki'):
                res.add(e)
                if len(res) > num and num > 0:
                    res.pop(0)
        return res

class EditWiki(Command):
    def execute(self, opts, wikihome):
        if self.input != None:
            path = self.input['path']
            page = self.mafs.open(wikihome + '/' + path + '.wiki')
            if page.exists:
                c = page.read()
                self.output = {
                    'path': path,
                    'content': c
                }
            else:
                self.output = {
                    'path': path,
                    'content': u''
                }
        else:
            self.output = {
                'path': u'',
                'content': u'',
            }
        return 0

class PostWiki(Command):
    def execute(self, opts, wikihome):
        name = self.input['path']
        wiki = wikihome.replace('/public', '')
        if '/' in name or '.' in name:
            self.output = {
                'header': {
                    'Location': self.config['server_url'] + wiki
                },
                'status': u'302 Found'
            }
            return -1
        path = wikihome + '/' + name + '.wiki'
        content = self.input['content']
        self.mafs.open(path, 'wb').write(content.encode('utf-8'))
        self.output = {
            'header': {
                'Location': self.config['server_url'] + wiki + '/' + name + '.wiki'
            },
            'status': u'302 Found'
        }
        return 0

class ViewWiki(Command):
    def execute(self, opts, path):
        f = self.mafs.open(path)
        t = f.read()
        c = creole2html(t)
        title = path.split('/')[-1].split('.')[0]
        self.output = {
            'link': path.split('/')[-1],
            'title': title,
            'body': c,
            'lastmod': str(f.last_modified)
        }
        return 0

